@@ -16,6 +16,26 @@ pub fn generate<T: Service>(service: &T, proto_path: &str) -> TokenStream {
     let service_doc = generate_doc_comments(service.comment());
     let (handlers, routes) = generate_handlers(service, proto_path);
 
+    let mut methods = TokenStream::new();
+
+    methods.extend(quote! {
+        /// Create a new service server.
+        pub fn new(service: T) -> Self {
+            Self {
+                service: Arc::new(service),
+            }
+        }
+    });
+
+    #[cfg(feature = "default_transport_http")]
+    methods.extend(quote! {
+        /// Serves the service with HTTP transport.
+        pub async fn serve<Addr: ToSocketAddrs>(self, addr: Addr) -> Result<(), <hrpc::server::transport::http::Hyper<Identity> as Transport>::Error> {
+            let transport = hrpc::server::transport::http::Hyper::new(addr)?;
+            transport.serve(self).await
+        }
+    });
+
     quote! {
         /// Generated server implementations.
         #[allow(unused_variables)]
@@ -46,18 +66,7 @@ pub fn generate<T: Service>(service: &T, proto_path: &str) -> TokenStream {
             }
 
             impl<T: #server_trait> #server_service<T> {
-                /// Create a new service server.
-                pub fn new(service: T) -> Self {
-                    Self {
-                        service: Arc::new(service),
-                    }
-                }
-
-                /// Serves the service with `hyper`.
-                pub async fn serve<Addr: ToSocketAddrs>(self, addr: Addr) -> Result<(), <Hyper<Addr> as Transport>::Error> {
-                    let transport = Hyper::new(addr);
-                    transport.serve(self).await
-                }
+                #methods
             }
         }
     }
@@ -93,7 +102,7 @@ fn generate_trait_methods<T: Service>(service: &T, proto_path: &str) -> TokenStr
         stream.extend(quote! {
             /// Optional middleware for this RPC.
             #[allow(unused_variables)]
-            fn #pre_name(&self, endpoint: &'static str) -> Option<HrpcLayer> {
+            fn #pre_name(&self) -> Option<HrpcLayer> {
                 None
             }
         });
@@ -105,7 +114,7 @@ fn generate_trait_methods<T: Service>(service: &T, proto_path: &str) -> TokenStr
             },
             (true, true) | (false, true) => quote! {
                 #method_doc
-                fn #name(&self, request: HrpcRequest<()>, socket: Socket<#req_message, #res_message>) -> BoxFuture<'_, ServerResult<()>>;
+                fn #name(&self, request: HrpcRequest<()>, socket: Socket<#res_message, #req_message>) -> BoxFuture<'_, ServerResult<()>>;
             },
             (true, false) => panic!("{}: Client streaming server unary method is invalid.", name),
         };
@@ -157,7 +166,7 @@ fn generate_handlers<T: Service>(service: &T, proto_path: &str) -> (TokenStream,
             (true, true) | (false, true) => {
                 quote! {
                     let svr = self.service.clone();
-                    let handler = move |request: HrpcRequest<()>, socket: Socket<#req_message, #res_message>| async move { svr. #name (request, socket).await };
+                    let handler = move |request: HrpcRequest<()>, socket: Socket<#res_message, #req_message>| async move { svr. #name (request, socket).await };
                     ws_handler(handler)
                 }
             }
@@ -168,7 +177,7 @@ fn generate_handlers<T: Service>(service: &T, proto_path: &str) -> (TokenStream,
             let #name = {
                 #handler_body
             };
-            let #name = match self.service. #pre_name (#endpoint) {
+            let #name = match self.service. #pre_name () {
                 Some(layer) => layer.layer(#name),
                 None => HrpcService::new(#name),
             };
